use std::fs;

use pest::Parser;
use pest_derive::Parser;

#[derive(Parser)]
#[grammar = "rpy.pest"]
struct RpyParser;

fn describe_token(pair: pest::iterators::Pair<Rule>) {
    let token = pair.as_rule();
    match token {
        Rule::token => {},
        _ => panic!("Not a token!"),
    };
    let contents = pair.into_inner().next().unwrap();
    let contents_rule = contents.as_rule();
    let str = match contents_rule {
        Rule::string => {
            let data = contents.into_inner().next().unwrap();
            match data.as_rule() {
                Rule::single_quote_string_data => data.as_str().replace("\\'", "'"),
                Rule::double_quote_string_data => data.as_str().replace("\\\"", "\""),
                _ => unreachable!(),
            }
        },
        Rule::array => {
            println!("array: Start array");
            for token in contents.into_inner() {
                describe_token(token);
            }
            "End array".to_string()
        }
        _ => contents.as_str().to_owned(),
    };
    println!("{:?}: {}", contents_rule, str);
}

pub fn parse(file_path: &str) {
    let unparsed_file = fs::read_to_string(file_path).expect("cannot find file");
    let file = RpyParser::parse(Rule::file, &unparsed_file)
        .expect("unsuccessful parse") // unwrap the parse result
        .next().unwrap(); // get and unwrap the `file` rule; never fails
    for line in file.into_inner() {
        match line.as_rule() {
            Rule::line => {
                for token in line.into_inner() {
                    describe_token(token);
                }
                println!()
            },
            Rule::EOI => (),
            _ => unreachable!(),
        }
    }
}
